/*
 * TismTool © Tis Veugen, 2010-2012
 * Simulation, Verification
 *
 * TsmGen.cs
 * version 18
 */
using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Runtime.Remoting.Contexts;

namespace TsmRTE
{
  public class TsmBase
  {
    public TsmBase TsmParent = null;
    public string TsmName = "";

    public virtual string TotalName()
    {
      string s = "";
      if ( null != TsmParent )
      {
        s = TsmParent.TotalName();
        string p = s.Substring( s.Length - 1, 1 );
        if ( p != ">" )
        {
          s += " ";
        }
        s += ">";
        if ( ! string.IsNullOrEmpty( TsmName ) )
        {
          s += " ";
        }
      }

      if ( this.TsmParent is TsmSubFsm )
      {
        return s + "{" + TsmName + "}";
      }
      return s + TsmName;
    }

    public virtual string FullName()
    {
      return "( " + TotalName() + " )";
    }
  }

  public class TsmPackage : TsmBase
  {
  }

  public class TsmException : Exception
  {
    public TsmException() { }

    public TsmException( string s )
      : base( s )
    {
      Tsm.Log( s );
    }
  }

  [Synchronization]
  public class Tsm
  {
    private static bool         mbLogConsole = false;
    private static bool         mbLogFile    = true;
    private static StreamWriter mLogWriter   = null;
    private static Object       mLogLock     = new Object();
    private static int          mLogLevel    = 1;
    private static bool         mbSeqFile    = true;
    private static StreamWriter mSeqWriter   = null;
    private static Object       mSeqLock     = new Object();
    private static int          mSeqClassId  = 0;

    public static void SetLogConsole( bool bLogConsole )
    {
      mbLogConsole = bLogConsole;
    }

    public static void SetLogFile( bool bLogFile )
    {
      mbLogFile = bLogFile;
    }

    public static void SetLogLevel( int iLevel )
    {
      mLogLevel = iLevel;
    }

    public static void SetSeqFile( bool bSeqFile )
    {
      mbSeqFile = bSeqFile;
    }

    public static void Assert( bool bCondition )
    {
      if ( ! bCondition )
      {
        Console.WriteLine( "ASSERT @ " + System.Environment.StackTrace );
        Tsm.Log( "ASSERT @ " + System.Environment.StackTrace );
      }
    } 

    public static void Open()
    {
      if ( mbLogFile )
      {
        string sFile = FreeTismFile( "Log" );
        FileStream fs = new FileStream( sFile, FileMode.Create, FileAccess.Write, FileShare.Write );
        mLogWriter = new StreamWriter( fs );
      }

      if ( mbSeqFile )
      {
        string sFile = FreeTismFile( "Seq" );
        FileStream fs = new FileStream( sFile, FileMode.Create, FileAccess.Write, FileShare.Write );
        mSeqWriter = new StreamWriter( fs );
        Seq( "TismTool, Sequence Diagrams" );
      }
    }

    public static void Log( string s )
    {
      lock( mLogLock )
      {
        if ( null != mLogWriter )
        {
          mLogWriter.WriteLine( s );
          mLogWriter.Flush();
        }

        if ( mbLogConsole )
        {
          Console.WriteLine( s );
        }
      }
    }

    public static void Lgl( int iLevel, string s )
    {
      if ( iLevel <= mLogLevel )
      {
        Log( s );
      }
    }

    public static int SeqClass( string s )
    {
      int Id;

      lock( mSeqLock )
      {
        Id = ++mSeqClassId;
      }

      Seq( "Class," + s + "," + Id );
      return Id;
    }

    public static void Seq( string s )
    {
      lock( mSeqLock )
      {
        if ( null != mSeqWriter )
        {
          mSeqWriter.WriteLine( s );
          mSeqWriter.Flush();
        }
      }
    }

    public static void ShowException( string s, Exception e )
    {
      Console.WriteLine( e.Message );
      Tsm.Lgl( 1, e.Message );
      Tsm.Lgl( 1, s + " : " + e.StackTrace );
    }

    public static void Close()
    {
      if ( null != mLogWriter )
      {
        mLogWriter.Close();
      }

      if ( null != mSeqWriter )
      {
        mSeqWriter.Close();
      }
    }

    private static string FreeTismFile( string sName )
    {
      string s;
      int ix = 0;

      do
      {
        ix++;
        s = string.Format( "Tism{0}_{1,4:D4}.txt", sName, ix );
      } while ( File.Exists( s ) );

      return s;
    }

    public static void ReportBox( string s )
    {
      Console.WriteLine( "Report: " + s );
    }
  }

  /**
   * Envelope of trigger message
   */
  public class TsmEnvelope
  {
    private TsmTrigger  mTrigger;
    private TsmClass    mTarget;

    public TsmEnvelope( TsmTrigger oTrigger, TsmClass oTarget )
    {
      this.mTrigger = oTrigger;
      this.mTarget  = oTarget;
    }

    public TsmTrigger Trigger
    {
      get { return mTrigger; }
    }

    public TsmClass Target
    {
      get { return mTarget; }
    }

    public string Snap()
    {
      return Trigger.Snap();
    }
  }

  /**
   * Rendez-vous states
   */
  public enum TsmRvState
  {
    IDLE,
    WAIT,
    READY
  }

  /**
   * Trigger message
   */
  public class TsmTrigger
  {
    public int          PortId;
    public int          Command;
    public TsmRvState   RvState = TsmRvState.IDLE;
    public bool         IsForeign = false;

    public TsmTrigger( int iCmd )
    {
      this.Command = iCmd;
    }

    public void Ready()
    {
      switch ( RvState )
      {
      case TsmRvState.IDLE:
        TsmRecorder.SyncReturnError( "**** RENDEZ-VOUS IDLE " );
        break;

      case TsmRvState.WAIT:
        RvState = TsmRvState.READY;
        break;

      case TsmRvState.READY:
        TsmRecorder.SyncReturnError( "**** RENDEZ-VOUS READY " );
        break;
      }
    }

    public string Snap()
    {
      return " " + PortId + "." + Command;
    }
  }
}
